@@ -3,7 +3,9 @@
 
 import pytest #This module contains unit tests.
 import unittest.mock #To monkeypatch some mocks in place of dependencies.
+import copy
 
+import cura.CuraApplication
 import cura.Settings.GlobalStack #The module we're testing.
 import cura.Settings.CuraContainerStack #To get the list of container types.
 from cura.Settings.Exceptions import TooManyExtrudersError, InvalidContainerError, InvalidOperationError #To test raising these errors.
@@ -13,6 +15,7 @@ from UM.Settings.SettingInstance import InstanceState
 import UM.Settings.ContainerRegistry
 import UM.Settings.ContainerStack
 import UM.Settings.SettingDefinition #To add settings to the definition.
+from UM.Settings.ContainerRegistry import ContainerRegistry
 
 ##  Fake container registry that always provides all containers you ask of.
 @pytest.yield_fixture()
@@ -33,6 +36,7 @@ def container_registry():
 #An empty global stack to test with.
 @pytest.fixture()
 def global_stack() -> cura.Settings.GlobalStack.GlobalStack:
+    creteEmptyContainers()
     return cura.Settings.GlobalStack.GlobalStack("TestStack")
 
 ##  Gets an instance container with a specified container type.
@@ -44,6 +48,31 @@ def getInstanceContainer(container_type) -> InstanceContainer:
     container.addMetaDataEntry("type", container_type)
     return container
 
+def creteEmptyContainers():
+    empty_container = ContainerRegistry.getInstance().getEmptyInstanceContainer()
+    empty_variant_container = copy.deepcopy(empty_container)
+    empty_variant_container.setMetaDataEntry("id", "empty_variant")
+    empty_variant_container.addMetaDataEntry("type", "variant")
+    ContainerRegistry.getInstance().addContainer(empty_variant_container)
+
+    empty_material_container = copy.deepcopy(empty_container)
+    empty_material_container.setMetaDataEntry("id", "empty_material")
+    empty_material_container.addMetaDataEntry("type", "material")
+    ContainerRegistry.getInstance().addContainer(empty_material_container)
+
+    empty_quality_container = copy.deepcopy(empty_container)
+    empty_quality_container.setMetaDataEntry("id", "empty_quality")
+    empty_quality_container.setName("Not Supported")
+    empty_quality_container.addMetaDataEntry("quality_type", "not_supported")
+    empty_quality_container.addMetaDataEntry("type", "quality")
+    empty_quality_container.addMetaDataEntry("supported", False)
+    ContainerRegistry.getInstance().addContainer(empty_quality_container)
+
+    empty_quality_changes_container = copy.deepcopy(empty_container)
+    empty_quality_changes_container.setMetaDataEntry("id", "empty_quality_changes")
+    empty_quality_changes_container.addMetaDataEntry("type", "quality_changes")
+    ContainerRegistry.getInstance().addContainer(empty_quality_changes_container)
+
 class DefinitionContainerSubClass(DefinitionContainer):
     def __init__(self):
         super().__init__(container_id = "SubDefinitionContainer")
@@ -87,31 +116,6 @@ def test_addExtruder(global_stack):
     #         global_stack.addExtruder(unittest.mock.MagicMock())
     assert len(global_stack.extruders) == 2 #Didn't add the faulty extruder.
 
-##  Tests getting the approximate material diameter.
-@pytest.mark.parametrize("diameter, approximate_diameter", [
-    #Some real-life cases that are common in printers.
-    (2.85, 3),
-    (1.75, 2),
-    (3.0, 3),
-    (2.0, 2),
-    #Exceptional cases.
-    (0, 0),
-    (-10.1, -10),
-    (-1, -1),
-    (9000.1, 9000)
-])
-def test_approximateMaterialDiameter(diameter, approximate_diameter, global_stack):
-    global_stack.definition = DefinitionContainer(container_id = "TestDefinition")
-    material_diameter = UM.Settings.SettingDefinition.SettingDefinition(key = "material_diameter", container = global_stack.definition)
-    material_diameter.addSupportedProperty("value", UM.Settings.SettingDefinition.DefinitionPropertyType.Any, default = diameter)
-    global_stack.definition.definitions.append(material_diameter)
-    assert float(global_stack.approximateMaterialDiameter) == approximate_diameter
-
-##  Tests getting the material diameter when there is no material diameter.
-def test_approximateMaterialDiameterNoDiameter(global_stack):
-    global_stack.definition = DefinitionContainer(container_id = "TestDefinition")
-    assert global_stack.approximateMaterialDiameter == "-1"
-
 #Tests setting user changes profiles to invalid containers.
 @pytest.mark.parametrize("container", [
     getInstanceContainer(container_type = "wrong container type"),
@@ -457,43 +461,6 @@ def test_removeContainer(global_stack):
     with pytest.raises(InvalidOperationError):
         global_stack.removeContainer(unittest.mock.MagicMock())
 
-##  Tests setting definitions by specifying an ID of a definition that exists.
-def test_setDefinitionByIdExists(global_stack, container_registry):
-    container_registry.return_value = DefinitionContainer(container_id = "some_definition")
-    global_stack.setDefinitionById("some_definition")
-    assert global_stack.definition.getId() == "some_definition"
-
-##  Tests setting definitions by specifying an ID of a definition that doesn't
-#   exist.
-def test_setDefinitionByIdDoesntExist(global_stack):
-    with pytest.raises(InvalidContainerError):
-        global_stack.setDefinitionById("some_definition") #Container registry is empty now.
-
-##  Tests setting definition changes by specifying an ID of a container that
-#   exists.
-def test_setDefinitionChangesByIdExists(global_stack, container_registry):
-    container_registry.return_value = getInstanceContainer(container_type = "definition_changes")
-    global_stack.setDefinitionChangesById("InstanceContainer")
-    assert global_stack.definitionChanges.getId() == "InstanceContainer"
-
-##  Tests setting definition changes by specifying an ID of a container that
-#   doesn't exist.
-def test_setDefinitionChangesByIdDoesntExist(global_stack):
-    with pytest.raises(InvalidContainerError):
-        global_stack.setDefinitionChangesById("some_definition_changes") #Container registry is empty now.
-
-##  Tests setting materials by specifying an ID of a material that exists.
-def test_setMaterialByIdExists(global_stack, container_registry):
-    container_registry.return_value = getInstanceContainer(container_type = "material")
-    global_stack.setMaterialById("InstanceContainer")
-    assert global_stack.material.getId() == "InstanceContainer"
-
-##  Tests setting materials by specifying an ID of a material that doesn't
-#   exist.
-def test_setMaterialByIdDoesntExist(global_stack):
-    with pytest.raises(InvalidContainerError):
-        global_stack.setMaterialById("some_material") #Container registry is empty now.
-
 ##  Tests whether changing the next stack is properly forbidden.
 def test_setNextStack(global_stack):
     with pytest.raises(InvalidOperationError):
@@ -538,50 +505,3 @@ def test_setPropertyOtherContainers(target_container, stack_variable, global_sta
     global_stack.setProperty(key, property, value, target_container = target_container) #The actual test.
 
     getattr(global_stack, stack_variable).setProperty.assert_called_once_with(key, property, value) #Make sure that the proper container gets a setProperty call.
-
-##  Tests setting qualities by specifying an ID of a quality that exists.
-def test_setQualityByIdExists(global_stack, container_registry):
-    container_registry.return_value = getInstanceContainer(container_type = "quality")
-    global_stack.setQualityById("InstanceContainer")
-    assert global_stack.quality.getId() == "InstanceContainer"
-
-##  Tests setting qualities by specifying an ID of a quality that doesn't exist.
-def test_setQualityByIdDoesntExist(global_stack):
-    with pytest.raises(InvalidContainerError):
-        global_stack.setQualityById("some_quality") #Container registry is empty now.
-
-##  Tests setting quality changes by specifying an ID of a quality change that
-#   exists.
-def test_setQualityChangesByIdExists(global_stack, container_registry):
-    container_registry.return_value = getInstanceContainer(container_type = "quality_changes")
-    global_stack.setQualityChangesById("InstanceContainer")
-    assert global_stack.qualityChanges.getId() == "InstanceContainer"
-
-##  Tests setting quality changes by specifying an ID of a quality change that
-#   doesn't exist.
-def test_setQualityChangesByIdDoesntExist(global_stack):
-    with pytest.raises(InvalidContainerError):
-        global_stack.setQualityChangesById("some_quality_changes") #Container registry is empty now.
-
-##  Tests setting variants by specifying an ID of a variant that exists.
-def test_setVariantByIdExists(global_stack, container_registry):
-    container_registry.return_value = getInstanceContainer(container_type = "variant")
-    global_stack.setVariantById("InstanceContainer")
-    assert global_stack.variant.getId() == "InstanceContainer"
-
-##  Tests setting variants by specifying an ID of a variant that doesn't exist.
-def test_setVariantByIdDoesntExist(global_stack):
-    with pytest.raises(InvalidContainerError):
-        global_stack.setVariantById("some_variant") #Container registry is empty now.
-
-##  Smoke test for findDefaultVariant
-def test_smoke_findDefaultVariant(global_stack):
-    global_stack.findDefaultVariant()
-
-##  Smoke test for findDefaultMaterial
-def test_smoke_findDefaultMaterial(global_stack):
-    global_stack.findDefaultMaterial()
-
-##  Smoke test for findDefaultQuality
-def test_smoke_findDefaultQuality(global_stack):
-    global_stack.findDefaultQuality()
